/*
 * This file is part of TONG.              http://www.nongnu.org/tong
 * Copyright 2004 Owen Swerkstrom
 *
 * TONG! is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TONG! is distributed in the hope that it will be useful and fun,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TONG!; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//TODO:  clean the hell out of this file, do convert surface on all gfx,
//       have pieces share one file/surface, new tetrad waits for break

#ifdef WIN32
#pragma comment(lib, "SDL.lib")
#pragma comment(lib, "SDLmain.lib")
#pragma comment(lib, "SDL_image.lib")
#pragma comment(lib, "SDL_mixer.lib")
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_image.h"
#include "tetris.h"
#include "pong.h"
#include "text.h"
#include "option.h"
#include "media.h"

#define KEY_DELAY           200
#define KEY_REPEAT          20

#define TITLE_SCREEN        0
#define GAME_SCREEN         1
#define OPTION_SCREEN       2
#define CONTROL_SCREEN      3
#define HISCORES_SCREEN     4
#define GAMEOVER_SCREEN     5

#define MENUMOUSE_THRESHOLD 40
#define DEMO_TIME           8000

bool drop_piece(Tetrad*, Tetrad*, Bucket*, Uint32&, bool&, bool&, Option*,
		int&);

//main graphics and sounds are global, hopefully that's not too horrible.
//globals are generally bad, but this is a game, after all.  :^)
SDL_Surface *back, *zone, *temp, *title, *scorebg, *timebg, *timeup;
SDL_Surface *nextbg, *rulebg, *ballbounce, *ballignore, *balljoin, *balldie;
SDL_Surface *tetradmove, *tetradignore, *tetradjoin, *tetradbreak, *tetraddie;
SDL_Surface *tetradrotate, *tetraddrop, *stackbreak, *stackignore, *stackjoin;
SDL_Surface *stackdie, *stacklinebreak, *gameover, *youlose;
SDL_Surface *icon;
Mix_Music *music;
Mix_Chunk *toing, *newrule, *twobits;
Mix_Chunk *rotate1, *rotate2, *plop, *bust, *tick;
Mix_Chunk *bounce1, *bounce2;


int main(int argc, char *argv[])
{
  SDL_Surface *screen;
  SDL_Joystick *joy=NULL;
  SDL_Event event;
  SDL_Event fakeinput;
  Uint32 lastupdate=0, lastlastupdate=0, lastpush=0;
  Uint32 lastrulechange=0, lastscoreblurb=0, demostarted=0;
  Uint32 lastbuttondown=0, lastbuttonup=0;
  bool repeatingbutton=false;
  int i=0, tempint=0;
  long int score=0;
  int display=TITLE_SCREEN;
  int xmasegg=0;
  bool gameinprogress=false;
  bool marathon=true;
  bool gothighscore=false;
  int scoredisplay=SCORE_MARATHON;
  Uint32 timeleft=0, endtime=0, timespent=0, starttime=0, musicstarted=0;
  Uint32 timelength=300000; // not60000, 120000, 300000, 600000
  char timetext[10]="";
  int pushes=0;
  int ballstatus;
  int ballslost=0, bucketslost=0;
  int done=0;
  int menumouse=0;
  bool wasbreaking=false;
  bool newdown=false;
  bool ruleupdated=false; //temp
  Sint16 pong_movex=0;
  Sint16 pong_movey=0;
  int level_tethover=0;
  int lines=0;
  char scoretext[20]="";
  char scorename[6]="_";
  Option *options;
  Text *scoreboard, *scoreblurb, *ruledisplay, *timedisplay, *highscores;
  Menu *mainmenu;
  Menu *highscoremenu;
  Bucket *bucket;
  Tetrad *tetrad, *nexttetrad;
  Paddle *paddle;
  Ball *ball;

  if(SDL_Init(SDL_INIT_AUDIO
	      |SDL_INIT_VIDEO
	      |SDL_INIT_TIMER
	      |SDL_INIT_JOYSTICK
	      )<0) {
    printf("Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }

  options=new Option();
  scoreboard=new Text();
  scoreblurb=new Text();
  ruledisplay=new Text();
  timedisplay=new Text();
  highscores=new Text();
  bucket=new Bucket();
  tetrad=new Tetrad();
  nexttetrad=new Tetrad();
  paddle=new Paddle();
  ball=new Ball();

  atexit(SDL_Quit);
  if(SDL_NumJoysticks()) {
    //printf("grabbing %s...\n", SDL_JoystickName(0));
    SDL_JoystickEventState(SDL_ENABLE);
    joy=SDL_JoystickOpen(0);
  }

  //let the randomness begin!
  srand((unsigned)time(NULL));

  options->ReadConfig();
  level_tethover=options->GetSpeedTetrad();

  screen=init_video(options->GetVideoBpp(), options->GetVideoFlags());
  load_graphics();

  SDL_EnableKeyRepeat(KEY_DELAY, KEY_REPEAT);
  SDL_ShowCursor(0);
  SDL_WM_GrabInput(SDL_GRAB_ON);
  SDL_WM_SetCaption("TONG!", NULL);
  SDL_WM_SetIcon(icon, NULL);

  init_audio(options, true, musicstarted);

  mainmenu=new Menu();
  mainmenu->SetPos(144, 176);
  mainmenu->AddChoice("Start Game!", NORMAL);
  mainmenu->AddChoice("Rules/Difficulty...", NORMAL);
  mainmenu->AddChoice("Controls...", NORMAL);
  mainmenu->AddChoice("Game Mode  = Marathon", NORMAL);
  mainmenu->AddChoice("Time Limit = 05:00", DISABLED);
  mainmenu->AddChoice("Graphics/Sound...", NORMAL);
  mainmenu->AddChoice("Quit", NORMAL);

  highscoremenu=new Menu();
  highscoremenu->SetPos(192, 192);
  highscoremenu->AddChoice("  HIGH SCORE!", NORMAL);
  highscoremenu->AddChoice("Enter your name", NORMAL);
  highscoremenu->AddChoice("     _", HIGHLIGHTED);
  highscoremenu->SetActive(true);
  highscoremenu->SetChoice(3);

  tetrad->Init(rand()%7);
  level_tethover=options->GetSpeedTetrad();
  nexttetrad->Init(rand()%7);
  bucket->Init(options->GetJunk());
  ball->Init(options->GetSpeedBall());
  paddle->Init(options->GetPadSizeX(), options->GetPadSizeY());
  timedisplay->Init(64, 80, "--:--.--");
  ruledisplay->Init(16, 16, "Welcome to TONG!");
  scoreblurb->Init(288, 352, "GO!!");
  scoreboard->Init(464, 80, "--------");
  highscores->Init(200, 24, options->GetHighScoreText(SCORE_MARATHON));
  mainmenu->SetActive(true);
  options->optionmenu->SetActive(true);
  lastupdate=SDL_GetTicks();
  demostarted=SDL_GetTicks();

  //get rid of any input that happened during initialization
  while(SDL_PollEvent(&event));

  while(!done) {
    if(display!=GAMEOVER_SCREEN) {
      if(options->GetMusic()==MUSIC_SHUFFLE && !Mix_PlayingMusic()) {
	Mix_FreeMusic(music);
	if((music=Mix_LoadMUS(options->GetMusicFile()))) {
	  Mix_PlayMusic(music, 1);
	}
	musicstarted=SDL_GetTicks();
      }
    }
    switch(display) {
    case TITLE_SCREEN:
      draw_bg(false, screen, options);
      draw_title(screen);
      mainmenu->Draw(screen);
      if(SDL_GetTicks()-demostarted>DEMO_TIME) {
	demostarted=SDL_GetTicks();
	display=HISCORES_SCREEN;
      }
      while(SDL_PollEvent(&event)) {
	demostarted=lastupdate;
	switch(event.type) {
	case SDL_QUIT:
	  done = 1;
	  break;
	case SDL_KEYDOWN:
	  switch(event.key.keysym.sym) {
	  case SDLK_ESCAPE:
	    if(options->GetSoundFx()) {
	      Mix_PlayChannel(0, toing, 0);
	    }
	    ball->UpdateTime();
	    if(!gameinprogress) {
	      mainmenu->ModChoice("Resume Game", 1);
	      mainmenu->ModChoice("End Current Game", 2);
	      mainmenu->SetState(4, DISABLED);
	      mainmenu->SetState(5, DISABLED);
	      options->RuleFrequencyInit();
	      timespent=0;
	    } else {
	      options->RuleFrequencyResume();
	    }
	    starttime=SDL_GetTicks()-timespent;
	    endtime=SDL_GetTicks()+timeleft;
	    display=GAME_SCREEN;
	    break;
	  case SDLK_UP:
	    if(mainmenu->MoveCursor(NORTH)) {
	      if(options->GetSoundFx()) {
		if(rand()%2) {
		  Mix_PlayChannel(0, rotate1, 0);
		} else {
		  Mix_PlayChannel(0, rotate2, 0);
		}
	      }
	    }
	    break;
	  case SDLK_DOWN:
	    if(mainmenu->MoveCursor(SOUTH)) {
	      if(options->GetSoundFx()) {
		if(rand()%2) {
		  Mix_PlayChannel(0, rotate1, 0);
		} else {
		  Mix_PlayChannel(0, rotate2, 0);
		}
	      }
	    }
	    break;
	  case SDLK_RETURN:
	    if(options->GetSoundFx()) {
	      Mix_PlayChannel(0, tick, 0);
	    }
	    switch(mainmenu->GetChoice()) {
	    case 1:
	      ball->UpdateTime();
	      if(!gameinprogress) {
		mainmenu->ModChoice("Resume Game", 1);
		mainmenu->ModChoice("End Current Game", 2);
		mainmenu->SetState(4, DISABLED);
		mainmenu->SetState(5, DISABLED);
		options->RuleFrequencyInit();
		timespent=0;
	      } else {
		options->RuleFrequencyResume();
	      }
	      endtime=SDL_GetTicks()+timeleft;
	      starttime=SDL_GetTicks()-timespent;
 	      timespent=0;
	      display=GAME_SCREEN;
	      break;
	    case 2:
	      if(gameinprogress) {
		gameinprogress=false;
		tetrad->Init(rand()%7);
		nexttetrad->Init(rand()%7);
		options->RuleFrequencyInit();
		bucket->Init(options->GetJunk());
		ball->Init(options->GetSpeedBall());
		paddle->Init(options->GetPadSizeX(), options->GetPadSizeY());
		ballslost=bucketslost=0;
		lines=0;
		lastrulechange=SDL_GetTicks();
		mainmenu->ModChoice("Start Game!", 1);
		mainmenu->ModChoice("Rules/Difficulty...", 2);
		mainmenu->SetState(4, NORMAL);
		if(!marathon) {
		  mainmenu->SetState(5, NORMAL);
		}
		ruledisplay->SetText("Welcome to TONG!");
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, plop, 0);
		}
		mainmenu->MoveCursor(NORTH);
	      } else {
		options->optionmenu->SetActive(false);
		options->custommenu->SetActive(true);
		display=OPTION_SCREEN;
	      }
	      break;
	    case 3:
	      display=CONTROL_SCREEN;
	      break;
	    case 4:
	      if(marathon) {
		mainmenu->ModChoice("Game Mode  = Timed   ", 4);
		mainmenu->SetState(5, NORMAL);
	      } else {
		mainmenu->ModChoice("Game Mode  = Marathon", 4);
		mainmenu->SetState(5, DISABLED);
	      }
	      marathon=!marathon;
	      break;
	    case 5:
	      switch(timelength) {
	      case 120000:
		timelength=300000;
		mainmenu->ModChoice("Time Limit = 05:00", 5);
		break;
	      case 300000:
		timelength=600000;
		mainmenu->ModChoice("Time Limit = 10:00", 5);
		break;
	      case 600000:
		timelength=120000;
		mainmenu->ModChoice("Time Limit = 02:00", 5);
	      default:
		break;
	      }
	      break;
	    case 6:
	      display=OPTION_SCREEN;
	      break;
	    case 7:
	      done=1;
	      break;
	    default:
	      break;
	    }
	    break;
	  case 'q':
	    done=1;
	    break;
	  case 'x':
	    if(xmasegg==0) {
	      xmasegg=1;
	    }
	    break;
	  case 'm':
	    if(xmasegg==1) {
	      xmasegg++;
	    } else {
	      xmasegg=0;
	    }
	    break;
	  case 'a':
	    if(xmasegg==2) {
	      xmasegg++;
	    } else {
	      xmasegg=0;
	    }
	    break;
	  case 's':
	    if(xmasegg==3) {
	      tetrad->Xmas();
	      nexttetrad->Xmas();
	      ball->Xmas();
	      paddle->Xmas();
	      bucket->Xmas();
	      Mix_PlayChannel(2, newrule, 0);
	      xmasegg++;
	    }
	    break;
	  case 'f':
	    options->SetVideoFlags(options->GetVideoFlags()^SDL_FULLSCREEN);
	    screen=init_video(options->GetVideoBpp(),
			      options->GetVideoFlags());
	    SDL_ShowCursor(0);
	    break;
	  default:
	    break;
	  }
	case SDL_MOUSEMOTION:
	  menumouse+=event.motion.yrel;
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  if(menumouse>MENUMOUSE_THRESHOLD) {
	    fakeinput.key.keysym.sym=SDLK_DOWN;
	    SDL_PushEvent(&fakeinput);
	    menumouse=0;
	  } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
	    fakeinput.key.keysym.sym=SDLK_UP;
	    SDL_PushEvent(&fakeinput);
	    menumouse=0;
	  }
	  break;
	case SDL_MOUSEBUTTONDOWN:
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  fakeinput.key.keysym.sym=SDLK_RETURN;
	  SDL_PushEvent(&fakeinput);
	  break;
	case SDL_JOYAXISMOTION:
	  switch(event.jaxis.axis) {
	  case 1: //up/down
	    fakeinput.type=SDL_KEYDOWN;
	    fakeinput.key.type=SDL_KEYDOWN;
	    fakeinput.key.state=SDL_PRESSED;
	    if(event.jaxis.value<-3200) {
	      fakeinput.key.keysym.sym=SDLK_UP;
	      SDL_PushEvent(&fakeinput);
	    } else if(event.jaxis.value>3200) {
	      fakeinput.key.keysym.sym=SDLK_DOWN;
	      SDL_PushEvent(&fakeinput);
	    }
	    break;
	  default:
	    break;
	  }
	  break;
	case SDL_JOYBUTTONDOWN:
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  fakeinput.key.keysym.sym=SDLK_RETURN;
	  SDL_PushEvent(&fakeinput);
	  break;
	}
      }
      break;
    case OPTION_SCREEN:
      draw_bg(false, screen, options);
      if(options->optionmenu->IsActive() || options->musicmenu->IsActive()) {
	options->optionmenu->Draw(screen);
	if(options->musicmenu->IsActive()) {
	  options->musicmenu->Draw(screen);
	  while(SDL_PollEvent(&event)) {
	    switch(event.type) {
	    case SDL_QUIT:
	      done = 1;
	      break;
	    case SDL_KEYDOWN:
	      switch(event.key.keysym.sym) {
	      case SDLK_ESCAPE:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		options->SetStart();
		options->optionmenu->SetActive(true);
		options->musicmenu->SetActive(false);
		break;
	      case SDLK_UP:
		if(options->musicmenu->MoveCursor(NORTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_DOWN:
		if(options->musicmenu->MoveCursor(SOUTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_RETURN:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		if(options->GetMusic()!=options->musicmenu->GetChoice()-1) {
		  options->SetMusic(options->musicmenu->GetChoice()-1);
		  Mix_FadeOutMusic(100);
		  Mix_FreeMusic(music);
		  if(options->GetMusicFile()) {
		    if((music=Mix_LoadMUS(options->GetMusicFile()))) {
		      if(options->GetMusic()==MUSIC_SHUFFLE) {
			if(music) {
			  Mix_PlayMusic(music, 1);
			}
		      } else {
			if(music) {
			  Mix_PlayMusic(music, -1);
			}
		      }
		      musicstarted=SDL_GetTicks();
		    } else {
		      printf("Loading music file %s failed.\n",
			     options->GetMusicFile());
		    }
		  } else {
		    //no music
		  }
		}
		options->optionmenu->SetActive(true);
		options->musicmenu->SetActive(false);
		break;
	      default:
		break;
	      }
	    case SDL_MOUSEMOTION:
	      menumouse+=event.motion.yrel;
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      if(menumouse>MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_DOWN;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_UP;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      }
	      break;
	    case SDL_MOUSEBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      fakeinput.key.keysym.sym=SDLK_RETURN;
	      SDL_PushEvent(&fakeinput);
	      break;
	    case SDL_JOYAXISMOTION:
	      switch(event.jaxis.axis) {
	      case 1: //up/down
		fakeinput.type=SDL_KEYDOWN;
		fakeinput.key.type=SDL_KEYDOWN;
		fakeinput.key.state=SDL_PRESSED;
		if(event.jaxis.value<-3200) {
		  fakeinput.key.keysym.sym=SDLK_UP;
		  SDL_PushEvent(&fakeinput);
		} else if(event.jaxis.value>3200) {
		  fakeinput.key.keysym.sym=SDLK_DOWN;
		  SDL_PushEvent(&fakeinput);
		}
		break;
	      default:
		break;
	      }
	      break;
	    case SDL_JOYBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      switch(event.jbutton.button) {
	      case 8: //start
		fakeinput.key.keysym.sym=SDLK_ESCAPE;
		SDL_PushEvent(&fakeinput);
		break;
	      case 1: //yellow/x/b
	      case 2: //red/square/y
	      default:
		fakeinput.key.keysym.sym=SDLK_RETURN;
		SDL_PushEvent(&fakeinput);
		break;
	      }
	      break;
	    default:
	      break;
	    }
	  }
	} else {
	  while(SDL_PollEvent(&event)) {
	    switch(event.type) {
	    case SDL_QUIT:
	      done = 1;
	      break;
	    case SDL_KEYDOWN:
	      switch(event.key.keysym.sym) {
	      case SDLK_ESCAPE:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, toing, 0);
		}
		demostarted=SDL_GetTicks();
		display=TITLE_SCREEN;
		break;
	      case SDLK_UP:
		if(options->optionmenu->MoveCursor(NORTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_DOWN:
		if(options->optionmenu->MoveCursor(SOUTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_RETURN:
		switch(options->optionmenu->GetChoice()) {
		case 1:
		  options->SetVideoFlags(options->GetVideoFlags()
					 ^SDL_FULLSCREEN);
		  screen=init_video(options->GetVideoBpp(),
				    options->GetVideoFlags());
		  SDL_ShowCursor(0);
		  break;
		case 2:
		  if(options->GetVideoBpp()>24) {
		    options->SetVideoBpp(16);
		  } else {
		    options->SetVideoBpp(options->GetVideoBpp()+8);
		  }
		  screen=init_video(options->GetVideoBpp(),
				    options->GetVideoFlags());
		  break;
		case 3:
		  if(options->GetAudioBuffers()>2048) {
		    options->SetAudioBuffers(512);
		  } else {
		    options->SetAudioBuffers(options->GetAudioBuffers()*2);
		  }
		  init_audio(options, false, musicstarted);
		  break;
		case 4:
		  if(options->GetAudioFreq()!=22050) {
		    options->SetAudioFreq(22050);
		  } else {
		    options->SetAudioFreq(44100);
		  }
		  init_audio(options, false, musicstarted);
		  break;
		case 5:
		  options->SetSoundFx(!options->GetSoundFx());
		  break;
		case 6:
		  options->optionmenu->SetActive(false);
		  options->musicmenu->SetActive(true);
		  break;
		case 7:
		  demostarted=SDL_GetTicks();
		  display=TITLE_SCREEN;
		  break;
		default:
		  break;
		}
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		break;
	      default:
		break;
	      }
	    case SDL_MOUSEMOTION:
	      menumouse+=event.motion.yrel;
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      if(menumouse>MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_DOWN;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_UP;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      }
	      break;
	    case SDL_MOUSEBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      fakeinput.key.keysym.sym=SDLK_RETURN;
	      SDL_PushEvent(&fakeinput);
	      break;
	    case SDL_JOYAXISMOTION:
	      switch(event.jaxis.axis) {
	      case 1: //up/down
		fakeinput.type=SDL_KEYDOWN;
		fakeinput.key.type=SDL_KEYDOWN;
		fakeinput.key.state=SDL_PRESSED;
		if(event.jaxis.value<-3200) {
		  fakeinput.key.keysym.sym=SDLK_UP;
		  SDL_PushEvent(&fakeinput);
		} else if(event.jaxis.value>3200) {
		  fakeinput.key.keysym.sym=SDLK_DOWN;
		  SDL_PushEvent(&fakeinput);
		}
		break;
	      default:
		break;
	      }
	      break;
	    case SDL_JOYBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      switch(event.jbutton.button) {
	      case 8: //start
		fakeinput.key.keysym.sym=SDLK_ESCAPE;
		SDL_PushEvent(&fakeinput);
		break;
	      case 1: //yellow/x/b
	      case 2: //red/square/y
	      default:
		fakeinput.key.keysym.sym=SDLK_RETURN;
		SDL_PushEvent(&fakeinput);
		break;
	      }
	      break;
	    default:
	      break;
	    }
	    break;
	  }
	}
      } else {
	options->custommenu->Draw(screen);
	if(options->ballstackmenu->IsActive()) {
	  options->ballstackmenu->Draw(screen);
	  while(SDL_PollEvent(&event)) {
	    switch(event.type) {
	    case SDL_QUIT:
	      done = 1;
	      break;
	    case SDL_KEYDOWN:
	      switch(event.key.keysym.sym) {
	      case SDLK_ESCAPE:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		options->SetStart();
		options->ballstackmenu->SetActive(false);
		options->custommenu->SetActive(true);
		break;
	      case SDLK_UP:
		if(options->ballstackmenu->MoveCursor(NORTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_DOWN:
		if(options->ballstackmenu->MoveCursor(SOUTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_RETURN:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		switch(options->ballstackmenu->GetChoice()) {
		case 1:
		  if(options->IsAllowed(NO_REACTION|RULETYPE_BALLSTACK) ||
		     options->IsAllowed(NO_REACTION|RULETYPE_STACKBALL)) {
		    options->DenyRule(NO_REACTION|RULETYPE_BALLSTACK);
		    options->DenyRule(NO_REACTION|RULETYPE_STACKBALL);
		  } else {
		    options->AllowRule(NO_REACTION|RULETYPE_BALLSTACK);
		    options->AllowRule(NO_REACTION|RULETYPE_STACKBALL);
		  }
		  break;
		case 2:
		  if(options->IsAllowed(STACK_KILLS_BALL|RULETYPE_STACKBALL)) {
		    options->DenyRule(STACK_KILLS_BALL|RULETYPE_STACKBALL);
		  } else {
		    options->AllowRule(STACK_KILLS_BALL|RULETYPE_STACKBALL);
		  }
		  break;
		case 3:
		  if(options->IsAllowed(STACK_BOUNCES_BALL
					|RULETYPE_STACKBALL)) {
		    options->DenyRule(STACK_BOUNCES_BALL|RULETYPE_STACKBALL);
		  } else {
		    options->AllowRule(STACK_BOUNCES_BALL|RULETYPE_STACKBALL);
		  }
		  break;
		case 4:
		  if(options->IsAllowed(BALL_JOINS_STACK|RULETYPE_BALLSTACK)) {
		    options->DenyRule(BALL_JOINS_STACK|RULETYPE_BALLSTACK);
		  } else {
		    options->AllowRule(BALL_JOINS_STACK|RULETYPE_BALLSTACK);
		  }
		  break;
		case 5:
		  if(options->IsAllowed(BALL_KILLS_STACK|RULETYPE_BALLSTACK)) {
		    options->DenyRule(BALL_KILLS_STACK|RULETYPE_BALLSTACK);
		  } else {
		    options->AllowRule(BALL_KILLS_STACK|RULETYPE_BALLSTACK);
		  }
		  break;
		case 6:
		  if(options->IsAllowed(BALL_BREAKS_STACK
					|RULETYPE_BALLSTACK)) {
		    options->DenyRule(BALL_BREAKS_STACK|RULETYPE_BALLSTACK);
		  } else {
		    options->AllowRule(BALL_BREAKS_STACK|RULETYPE_BALLSTACK);
		  }
		  break;
		case 7:
		  if(options->IsAllowed(BALL_LINEBREAK_STACK
					|RULETYPE_BALLSTACK)) {
		    options->DenyRule(BALL_LINEBREAK_STACK|RULETYPE_BALLSTACK);
		  } else {
		    options->AllowRule(BALL_LINEBREAK_STACK
				       |RULETYPE_BALLSTACK);
		  }
		  break;
		case 8:
		  options->SetStart();
		  options->ballstackmenu->SetActive(false);
		  options->custommenu->SetActive(true);
		  break;
		default:
		  break;
		}
		break;
	      default:
		break;
	      }
	    case SDL_MOUSEMOTION:
	      menumouse+=event.motion.yrel;
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      if(menumouse>MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_DOWN;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_UP;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      }
	      break;
	    case SDL_MOUSEBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      fakeinput.key.keysym.sym=SDLK_RETURN;
	      SDL_PushEvent(&fakeinput);
	      break;
	    case SDL_JOYAXISMOTION:
	      switch(event.jaxis.axis) {
	      case 1: //up/down
		fakeinput.type=SDL_KEYDOWN;
		fakeinput.key.type=SDL_KEYDOWN;
		fakeinput.key.state=SDL_PRESSED;
		if(event.jaxis.value<-3200) {
		  fakeinput.key.keysym.sym=SDLK_UP;
		  SDL_PushEvent(&fakeinput);
		} else if(event.jaxis.value>3200) {
		  fakeinput.key.keysym.sym=SDLK_DOWN;
		  SDL_PushEvent(&fakeinput);
		}
		break;
	      default:
		break;
	      }
	      break;
	    case SDL_JOYBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      switch(event.jbutton.button) {
	      case 8: //start
		fakeinput.key.keysym.sym=SDLK_ESCAPE;
		SDL_PushEvent(&fakeinput);
		break;
	      case 1: //yellow/x/b
	      case 2: //red/square/y
	      default:
		fakeinput.key.keysym.sym=SDLK_RETURN;
		SDL_PushEvent(&fakeinput);
		break;
	      }
	      break;
	    default:
	      break;
	    }
	  }
	} else if(options->balltetradmenu->IsActive()) {
	  options->balltetradmenu->Draw(screen);
	  while(SDL_PollEvent(&event)) {
	    switch(event.type) {
	    case SDL_QUIT:
	      done = 1;
	      break;
	    case SDL_KEYDOWN:
	      switch(event.key.keysym.sym) {
	      case SDLK_ESCAPE:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		options->SetStart();
		options->balltetradmenu->SetActive(false);
		options->custommenu->SetActive(true);
		break;
	      case SDLK_UP:
		if(options->balltetradmenu->MoveCursor(NORTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_DOWN:
		if(options->balltetradmenu->MoveCursor(SOUTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_RETURN:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		switch(options->balltetradmenu->GetChoice()) {
		case 1:
		  if(options->IsAllowed(NO_REACTION|RULETYPE_BALLTETRAD) ||
		     options->IsAllowed(NO_REACTION|RULETYPE_TETRADBALL)) {
		    options->DenyRule(NO_REACTION|RULETYPE_BALLTETRAD);
		    options->DenyRule(NO_REACTION|RULETYPE_TETRADBALL);
		  } else {
		    options->AllowRule(NO_REACTION|RULETYPE_BALLTETRAD);
		    options->AllowRule(NO_REACTION|RULETYPE_TETRADBALL);
		  }
		  break;
		case 2:
		  if(options->IsAllowed(TETRAD_KILLS_BALL
					|RULETYPE_TETRADBALL)) {
		    options->DenyRule(TETRAD_KILLS_BALL|RULETYPE_TETRADBALL);
		  } else {
		    options->AllowRule(TETRAD_KILLS_BALL|RULETYPE_TETRADBALL);
		  }
		  break;
		case 3:
		  if(options->IsAllowed(TETRAD_BOUNCES_BALL
					|RULETYPE_TETRADBALL)) {
		    options->DenyRule(TETRAD_BOUNCES_BALL|RULETYPE_TETRADBALL);
		  } else {
		    options->AllowRule(TETRAD_BOUNCES_BALL
				       |RULETYPE_TETRADBALL);
		  }
		  break;
		case 4:
		  if(options->IsAllowed(BALL_KILLS_TETRAD
					|RULETYPE_BALLTETRAD)) {
		    options->DenyRule(BALL_KILLS_TETRAD|RULETYPE_BALLTETRAD);
		  } else {
		    options->AllowRule(BALL_KILLS_TETRAD|RULETYPE_BALLTETRAD);
		  }
		  break;
		case 5:
		  if(options->IsAllowed(BALL_JOINS_TETRAD
					|RULETYPE_BALLTETRAD)) {
		    options->DenyRule(BALL_JOINS_TETRAD|RULETYPE_BALLTETRAD);
		  } else {
		    options->AllowRule(BALL_JOINS_TETRAD|RULETYPE_BALLTETRAD);
		  }
		  break;
		case 6:
		  if(options->IsAllowed(BALL_BREAKS_TETRAD
					|RULETYPE_BALLTETRAD)) {
		    options->DenyRule(BALL_BREAKS_TETRAD|RULETYPE_BALLTETRAD);
		  } else {
		    options->AllowRule(BALL_BREAKS_TETRAD|RULETYPE_BALLTETRAD);
		  }
		  break;
		case 7:
		  if(options->IsAllowed(BALL_DROPS_TETRAD
					|RULETYPE_BALLTETRAD)) {
		    options->DenyRule(BALL_DROPS_TETRAD|RULETYPE_BALLTETRAD);
		  } else {
		    options->AllowRule(BALL_DROPS_TETRAD|RULETYPE_BALLTETRAD);
		  }
		  break;
		case 8:
		  if(options->IsAllowed(BALL_MOVES_TETRAD
					|RULETYPE_BALLTETRAD)) {
		    options->DenyRule(BALL_MOVES_TETRAD|RULETYPE_BALLTETRAD);
		  } else {
		    options->AllowRule(BALL_MOVES_TETRAD|RULETYPE_BALLTETRAD);
		  }
		  break;
		case 9:
		  if(options->IsAllowed(BALL_ROTATES_TETRAD
					|RULETYPE_BALLTETRAD)) {
		    options->DenyRule(BALL_ROTATES_TETRAD|RULETYPE_BALLTETRAD);
		  } else {
		    options->AllowRule(BALL_ROTATES_TETRAD
				       |RULETYPE_BALLTETRAD);
		  }
		  break;
		case 10:
		  options->SetStart();
		  options->balltetradmenu->SetActive(false);
		  options->custommenu->SetActive(true);
		  break;
		default:
		  break;
		}
		break;
	      default:
		break;
	      }
	    case SDL_MOUSEMOTION:
	      menumouse+=event.motion.yrel;
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      if(menumouse>MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_DOWN;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_UP;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      }
	      break;
	    case SDL_MOUSEBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      fakeinput.key.keysym.sym=SDLK_RETURN;
	      SDL_PushEvent(&fakeinput);
	      break;
	    case SDL_JOYAXISMOTION:
	      switch(event.jaxis.axis) {
	      case 1: //up/down
		fakeinput.type=SDL_KEYDOWN;
		fakeinput.key.type=SDL_KEYDOWN;
		fakeinput.key.state=SDL_PRESSED;
		if(event.jaxis.value<-3200) {
		  fakeinput.key.keysym.sym=SDLK_UP;
		  SDL_PushEvent(&fakeinput);
		} else if(event.jaxis.value>3200) {
		  fakeinput.key.keysym.sym=SDLK_DOWN;
		  SDL_PushEvent(&fakeinput);
		}
		break;
	      default:
		break;
	      }
	      break;
	    case SDL_JOYBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      switch(event.jbutton.button) {
	      case 8: //start
		fakeinput.key.keysym.sym=SDLK_ESCAPE;
		SDL_PushEvent(&fakeinput);
		break;
	      case 1: //yellow/x/b
	      case 2: //red/square/y
	      default:
		fakeinput.key.keysym.sym=SDLK_RETURN;
		SDL_PushEvent(&fakeinput);
		break;
	      }
	      break;
	    default:
	      break;
	    }
	  }
	} else {
	  while(SDL_PollEvent(&event)) {
	    switch(event.type) {
	    case SDL_QUIT:
	      done = 1;
	      break;
	    case SDL_KEYDOWN:
	      switch(event.key.keysym.sym) {
	      case SDLK_ESCAPE:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, toing, 0);
		}
		demostarted=SDL_GetTicks();
		display=TITLE_SCREEN;
		break;
	      case SDLK_UP:
		if(options->custommenu->MoveCursor(NORTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_DOWN:
		if(options->custommenu->MoveCursor(SOUTH)) {
		  if(options->GetSoundFx()) {
		    if(rand()%2) {
		      Mix_PlayChannel(0, rotate1, 0);
		    } else {
		      Mix_PlayChannel(0, rotate2, 0);
		    }
		  }
		}
		break;
	      case SDLK_RETURN:
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, tick, 0);
		}
		switch(options->custommenu->GetChoice()) {
		case 1:
		  /*
		  options->DefaultRules();
		  options->DefaultSettings();
		  options->SetStart();
		  */
		  options->SetLevel();
		  level_tethover=options->GetSpeedTetrad();
		  ball->Init(options->GetSpeedBall());
		  paddle->Init(options->GetPadSizeX(), options->GetPadSizeY());
		  bucket->Init(options->GetJunk());
		  break;
		case 2:
		  switch(options->GetPadPlacement()) {
		  case PADS_ALL:
		    options->SetPadPlacement(PADS_SIDES);
		    break;
		  case PADS_SIDES:
		    options->SetPadPlacement(PADS_TOPBOT);
		    break;
		  case PADS_TOPBOT:
		  default:
		    options->SetPadPlacement(PADS_ALL);
		    break;
		  }
		  paddle->Init(options->GetPadSizeX(),
			       options->GetPadSizeY());
		  break;
		case 3:
		  switch(options->GetPadSize()) {
		  case PADS_SMALL:
		    options->SetPadSize(PADS_NORMAL);
		    break;
		  case PADS_NORMAL:
		    options->SetPadSize(PADS_BIG);
		    break;
		  case PADS_BIG:
		    options->SetPadSize(PADS_15);
		    break;
		  case PADS_15:
		    options->SetPadSize(PADS_13);
		    break;
		  case PADS_13:
		    options->SetPadSize(PADS_12);
		    break;
		  case PADS_12:
		  default:
		    options->SetPadSize(PADS_SMALL);
		    break;
		  }
		  paddle->Init(options->GetPadSizeX(),
			       options->GetPadSizeY());
		  break;
		case 4:
		  options->SetTetSpeedUp(!options->GetTetSpeedUp());
		  break;
		case 5:
		  options->SetBallSpeedsUp(!options->GetBallSpeedsUp());
		  break;
		case 6:
		  options->SetRandomRule(!options->GetRandomRule());
		  break;
		case 7:
		  switch(options->GetRuleFrequency()) {
		  case EVERY_0_30:
		    options->SetRuleFrequency(EVERY_0_45);
		    break;
		  case EVERY_0_45:
		    options->SetRuleFrequency(EVERY_1_00);
		    break;
		  case EVERY_1_00:
		    options->SetRuleFrequency(EVERY_BALL);
		    break;
		  case EVERY_BALL:
		    options->SetRuleFrequency(EVERY_3_BALLS);
		    break;
		  case EVERY_3_BALLS:
		    options->SetRuleFrequency(EVERY_LINE);
		    break;
		  case EVERY_LINE:
		    options->SetRuleFrequency(EVERY_5_LINES);
		    break;
		  case EVERY_5_LINES:
		    options->SetRuleFrequency(EVERY_0_15);
		    break;
		  case EVERY_0_15:
		  default:
		    options->SetRuleFrequency(EVERY_0_30);
		    break;
		  }
		  break;
		case 8:
		  options->SetSpeed();
		  break;
		case 9:
		  options->SetJunk();
		  bucket->Init(options->GetJunk());
		  break;
		case 10:
		  options->custommenu->SetActive(false);
		  options->balltetradmenu->SetActive(true);
		  break;
		case 11:
		  options->custommenu->SetActive(false);
		  options->ballstackmenu->SetActive(true);
		  break;
		case 12:
		  options->custommenu->SetActive(false);
		  options->optionmenu->SetActive(true);
		  mainmenu->SetActive(true);
		  demostarted=SDL_GetTicks();
		  display=TITLE_SCREEN;
		  break;
		default:
		  break;
		}
		break;
	      default:
		break;
	      }
	    case SDL_MOUSEMOTION:
	      menumouse+=event.motion.yrel;
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      if(menumouse>MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_DOWN;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
		fakeinput.key.keysym.sym=SDLK_UP;
		SDL_PushEvent(&fakeinput);
		menumouse=0;
	      }
	      break;
	    case SDL_MOUSEBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      fakeinput.key.keysym.sym=SDLK_RETURN;
	      SDL_PushEvent(&fakeinput);
	      break;
	    case SDL_JOYAXISMOTION:
	      switch(event.jaxis.axis) {
	      case 1: //up/down
		fakeinput.type=SDL_KEYDOWN;
		fakeinput.key.type=SDL_KEYDOWN;
		fakeinput.key.state=SDL_PRESSED;
		if(event.jaxis.value<-3200) {
		  fakeinput.key.keysym.sym=SDLK_UP;
		  SDL_PushEvent(&fakeinput);
		} else if(event.jaxis.value>3200) {
		  fakeinput.key.keysym.sym=SDLK_DOWN;
		  SDL_PushEvent(&fakeinput);
		}
		break;
	      default:
		break;
	      }
	      break;
	    case SDL_JOYBUTTONDOWN:
	      fakeinput.type=SDL_KEYDOWN;
	      fakeinput.key.type=SDL_KEYDOWN;
	      fakeinput.key.state=SDL_PRESSED;
	      switch(event.jbutton.button) {
	      case 8: //start
		fakeinput.key.keysym.sym=SDLK_ESCAPE;
		SDL_PushEvent(&fakeinput);
		break;
	      case 1: //yellow/x/b
	      case 2: //red/square/y
	      default:
		fakeinput.key.keysym.sym=SDLK_RETURN;
		SDL_PushEvent(&fakeinput);
		break;
	      }
	      break;
	    default:
	      break;
	    }
	  } //event poll loop
	}
      }
      break;
    case CONTROL_SCREEN:
      draw_bg(false, screen, options);
      options->controlmenu->Draw(screen);
      while(SDL_PollEvent(&event)) {
	switch(event.type) {
	case SDL_QUIT:
	  done = 1;
	  break;
	case SDL_KEYDOWN:
	  switch(event.key.keysym.sym) {
	  case SDLK_ESCAPE:
	    if(options->GetSoundFx()) {
	      Mix_PlayChannel(0, toing, 0);
	    }
	    if(options->IsSettingKey()) {
	      options->CancelSetKey();
	    } else {
	      demostarted=SDL_GetTicks();
	      display=TITLE_SCREEN;
	    }
	    break;
	  case SDLK_UP:
	    if(options->IsSettingKey()) {
	      options->SetKey(event.key.keysym.sym);
	    } else {
	      if(options->controlmenu->MoveCursor(NORTH)) {
		if(options->GetSoundFx()) {
		  if(rand()%2) {
		    Mix_PlayChannel(0, rotate1, 0);
		  } else {
		    Mix_PlayChannel(0, rotate2, 0);
		  }
		}
	      }
	    }
	    break;
	  case SDLK_DOWN:
	    if(options->IsSettingKey()) {
	      options->SetKey(event.key.keysym.sym);
	    } else {
	      if(options->controlmenu->MoveCursor(SOUTH)) {
		if(options->GetSoundFx()) {
		  if(rand()%2) {
		    Mix_PlayChannel(0, rotate1, 0);
		  } else {
		    Mix_PlayChannel(0, rotate2, 0);
		  }
		}
	      }
	    }
	    break;
	  case SDLK_RETURN:
	    if(options->IsSettingKey()) {
	      options->CancelSetKey();
	    } else {
	      if(options->GetSoundFx()) {
		Mix_PlayChannel(0, tick, 0);
	      }
	      switch(options->controlmenu->GetChoice()) {
	      case 1:
		options->DefaultControls();
		break;
	      case 8:
		demostarted=SDL_GetTicks();
		display=TITLE_SCREEN;
		break;
	      default:
		options->SetKey();
		break;
	      }
	    }
	    break;
	  default:
	    if(options->IsSettingKey()) {
	      options->SetKey(event.key.keysym.sym);
	    }
	    break;
	  }
	case SDL_MOUSEMOTION:
	  menumouse+=event.motion.yrel;
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  if(menumouse>MENUMOUSE_THRESHOLD) {
	    fakeinput.key.keysym.sym=SDLK_DOWN;
	    SDL_PushEvent(&fakeinput);
	    menumouse=0;
	  } else if(menumouse<0-MENUMOUSE_THRESHOLD) {
	    fakeinput.key.keysym.sym=SDLK_UP;
	    SDL_PushEvent(&fakeinput);
	    menumouse=0;
	  }
	  break;
	case SDL_MOUSEBUTTONDOWN:
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  fakeinput.key.keysym.sym=SDLK_RETURN;
	  SDL_PushEvent(&fakeinput);
	  break;
	case SDL_JOYAXISMOTION:
	  switch(event.jaxis.axis) {
	  case 1: //up/down
	    fakeinput.type=SDL_KEYDOWN;
	    fakeinput.key.type=SDL_KEYDOWN;
	    fakeinput.key.state=SDL_PRESSED;
	    if(event.jaxis.value<-3200) {
	      fakeinput.key.keysym.sym=SDLK_UP;
	      SDL_PushEvent(&fakeinput);
	    } else if(event.jaxis.value>3200) {
	      fakeinput.key.keysym.sym=SDLK_DOWN;
	      SDL_PushEvent(&fakeinput);
	    }
	    break;
	  default:
	    break;
	  }
	  break;
	case SDL_JOYBUTTONDOWN:
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  switch(event.jbutton.button) {
	  case 8: //start
	    fakeinput.key.keysym.sym=SDLK_ESCAPE;
	    SDL_PushEvent(&fakeinput);
	    break;
	  case 1: //yellow/x/b
	  case 2: //red/square/y
	  default:
	    fakeinput.key.keysym.sym=SDLK_RETURN;
	    SDL_PushEvent(&fakeinput);
	    break;
	  }
	  break;
	}
      }
      break;
    case GAME_SCREEN:
      if(gameinprogress) {
	//pick up where we left off
      } else {
	//or start over, you know, whatever
	lastrulechange=SDL_GetTicks();
	options->SetStart();
	score=0;
	endtime=SDL_GetTicks()+timelength;
	starttime=SDL_GetTicks();
	timespent=0;
	gameinprogress=true;
      }

      if(!marathon && SDL_GetTicks()>endtime) {
	display=GAMEOVER_SCREEN;
	Mix_FadeOutMusic(1000);
	bucket->GreyOut();
	if(options->GetSoundFx()) {
	  Mix_PlayChannel(2, twobits, 0);
	}
      }

      if(options->RuleFrequencyNow()) {
	if(options->GetSoundFx()) {
	  Mix_PlayChannel(2, newrule, 0);
	}
	options->NewRule();
	ruledisplay->SetText(options->GetRuleText());
	lastrulechange=lastupdate;
	ruleupdated=true;
      }
      ballstatus=ball->Move(paddle, bucket, tetrad, nexttetrad, options);
      if(ballstatus==OUT_OF_BOUNDS) {
	ball->Init(options->GetSpeedBall());
	options->RuleFrequencyAddBall();
	if(marathon) {
	  tetrad->MoveUp(bucket);
	  bucket->AddJunk();
	  drop_piece(tetrad, nexttetrad, bucket, lastpush, newdown, marathon,
		     options, display);
	  //we move the tetrad up and down around the junk adding so it
	  //does not get embedded if it's touching during the rise
	  score+=1;
	} else {
	  tempint=100*(++ballslost);
	  sprintf(scoretext, "-%i", tempint);
	  score-=tempint-1;
	  scoreblurb->SetText(scoretext);
	  lastscoreblurb=lastupdate;
	}
      } else if(ballstatus==BOUNCED) {
	tempint=100;
	/*
	sprintf(scoretext, "+%i", tempint);
	scoreblurb->SetText(scoretext);
	lastscoreblurb=lastupdate;
	*/
	score+=tempint;
	if(options->GetSoundFx()) {
	  if(rand()%2) {
	    Mix_PlayChannel(1, bounce1, 0);
	  } else {
	    Mix_PlayChannel(1, bounce2, 0);
	  }
	}
      }
      if(pong_movex) {
	paddle->MoveX(pong_movex);
      }
      if(pong_movey) {
	paddle->MoveY(pong_movey);
      }
      
      if(wasbreaking) {
	wasbreaking=bucket->IsBreaking();
	if(!wasbreaking && tetrad->HasLanded()) {
	  tetrad->Init(nexttetrad->GetType());
	  nexttetrad->Init(rand()%7);
	}
      } else {
	wasbreaking=bucket->IsBreaking();
	if(wasbreaking && options->GetSoundFx()) {
	  Mix_PlayChannel(0, bust, 0);
	}
      }
      //push down every second or so, according to level
      //TODO:  uber-slow machines would not get enough pushes over time
      if(SDL_GetTicks()>lastpush+level_tethover) {
	if(drop_piece(tetrad, nexttetrad, bucket, lastpush, newdown, marathon,
		      options, display)) {
	  tempint=5000*(++bucketslost);
	  sprintf(scoretext, "-%i", tempint);
	  scoreblurb->SetText(scoretext);
	  lastscoreblurb=lastupdate;
	  score-=tempint-1;
	}
      }

      if(lastbuttondown>lastbuttonup) {
	if(repeatingbutton) {
	  if(lastbuttondown+KEY_REPEAT<lastupdate) {
	    SDL_PushEvent(&fakeinput);
	  }
	} else {
	  if(lastbuttondown+KEY_DELAY<lastupdate) {
	    SDL_PushEvent(&fakeinput);
	    repeatingbutton=true;
	  }
	}
      }
      
      while(SDL_PollEvent(&event)) {
	switch(event.type) {
	case SDL_QUIT:
	  done = 1;
	  break;
	case SDL_KEYUP:
	  newdown=false;
	  pushes=0;
	  break;
	case SDL_JOYAXISMOTION:
	  switch(event.jaxis.axis) {
	  case 0: //left/right
	    pong_movex=event.jaxis.value;
	    break;
	  case 1: //up/down
	  default:
	    pong_movey=event.jaxis.value;
	    break;
	  }
	  break;
	case SDL_JOYBUTTONUP:
	  pushes=0;
	  newdown=false;
	  lastbuttonup=lastupdate;
	  repeatingbutton=false;
	  break;
	case SDL_JOYBUTTONDOWN:
	  lastbuttondown=lastupdate;
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  switch(event.jbutton.button) {
	  case 0: //green/circle/a?
	    fakeinput.key.keysym.sym=options->GetKey(KEY_TETRIGHT);
	    SDL_PushEvent(&fakeinput);
	    break;
	  case 1: //yellow/x/b
	    fakeinput.key.keysym.sym=options->GetKey(KEY_TETDOWN);
	    SDL_PushEvent(&fakeinput);
	    break;
	  case 2: //red/square/y
	    fakeinput.key.keysym.sym=options->GetKey(KEY_TETLEFT);
	    SDL_PushEvent(&fakeinput);
	    break;
	  case 3: //blue/triangle/x?
	    fakeinput.key.keysym.sym=options->GetKey(KEY_TETCW);
	    SDL_PushEvent(&fakeinput);
	    break;
	  case 8: //start
	    fakeinput.key.keysym.sym=SDLK_ESCAPE;
	    SDL_PushEvent(&fakeinput);
	    break;
	  default:
	    fakeinput.key.keysym.sym=SDLK_RETURN;
	    SDL_PushEvent(&fakeinput);
	    break;
	  }
	  break;
	case SDL_KEYDOWN:
	  if(event.key.keysym.sym==SDLK_ESCAPE) {
	    if(options->GetSoundFx()) {
	      Mix_PlayChannel(0, toing, 0);
	    }
	    timeleft=endtime-SDL_GetTicks();
	    timespent=SDL_GetTicks()-starttime;
	    options->RuleFrequencyPause();
	    demostarted=SDL_GetTicks();
	    display=TITLE_SCREEN;
	  } else if(event.key.keysym.sym==options->GetKey(KEY_TETRIGHT)) {
	    if(pushes!=0) {
	      pushes=(lastupdate-lastlastupdate)/KEY_REPEAT;
	    }
	    pushes=pushes?pushes:1;
	    for(i=0; i<pushes; i++) {
	      if(tetrad->MoveRight(bucket) && options->GetSoundFx()) {
		Mix_PlayChannel(0, tick, 0);
	      }
	    }
	  } else if(event.key.keysym.sym==options->GetKey(KEY_TETLEFT)) {
	    if(pushes!=0) {
	      pushes=(lastupdate-lastlastupdate)/KEY_REPEAT;
	    }
	    pushes=pushes?pushes:1;
	    for(i=0; i<pushes; i++) {
	      if(tetrad->MoveLeft(bucket) && options->GetSoundFx()) {
		Mix_PlayChannel(0, tick, 0);
	      }
	    }
	  } else if(event.key.keysym.sym==options->GetKey(KEY_TETDOWN)) {
	    if(!newdown) {
	      if(pushes!=0) {
		pushes=(lastupdate-lastlastupdate)/KEY_REPEAT;
	      }
	      pushes=pushes?pushes:1;
	      //keeps pushes the same or turns to 1 if it was/got 0
	      for(i=0; i<pushes; i++) {
		if(drop_piece(tetrad, nexttetrad, bucket, lastpush, newdown,
			      marathon, options, display)) {
		  tempint=5000*(++bucketslost);
		  sprintf(scoretext, "-%i", tempint);
		  scoreblurb->SetText(scoretext);
		  lastscoreblurb=lastupdate;
		  score-=tempint-1;
		} else {
		  if(newdown) {
		    break;
		  }
		}
	      }
	    } else {
	    }
	  } else if(event.key.keysym.sym==options->GetKey(KEY_TETCW)) {
	    if(tetrad->Rotate(bucket, true)) {
	      if(options->GetSoundFx()) {
		if(rand()%2) {
		  Mix_PlayChannel(0, rotate1, 0);
		} else {
		  Mix_PlayChannel(0, rotate2, 0);
		}
	      }
	    }
	  } else if(event.key.keysym.sym==options->GetKey(KEY_TETCCW)) {
	    if(tetrad->Rotate(bucket, false)) {
	      if(rand()%2) {
		if(options->GetSoundFx()) {
		  Mix_PlayChannel(0, rotate1, 0);
		} else {
		  Mix_PlayChannel(0, rotate2, 0);
		}
	      }
	    }
	  } else if(event.key.keysym.sym==SDLK_PRINT) {
	    SDL_SaveBMP(screen, "screen.bmp");
	  } else {
	  }
	  break;
	case SDL_MOUSEMOTION:
	  paddle->MouseMoved(event.motion.xrel, event.motion.yrel);
	  break;
	default:
	  break;
	}
      }

      draw_bg(true, screen, options);
      tetrad->Draw(screen);
      nexttetrad->Preview(screen);
      tempint=bucket->Draw(screen);
      paddle->Draw(screen);
      ball->Draw(screen);

      if(marathon) {
	sprintf(timetext, "%02i:%02i.%02i",
		(SDL_GetTicks()-starttime)/1000/60,
		((SDL_GetTicks()-starttime)/1000)%60,
		((SDL_GetTicks()-starttime)/10)%100);
      } else {
	sprintf(timetext, "%02i:%02i.%02i",
		(endtime-SDL_GetTicks())/1000/60,
		((endtime-SDL_GetTicks())/1000)%60,
		((endtime-SDL_GetTicks())/10)%100);
      }
      timedisplay->SetText(timetext);
      timedisplay->Draw(screen);
      if(lastscoreblurb && lastupdate-lastscoreblurb<1500) {
	scoreblurb->Draw(screen);
      }
      if(lastrulechange && lastupdate-lastrulechange<4000) {
	ruledisplay->Draw(screen);
      }
      //line(s) broken
      if(tempint) {
	if(options->GetTetSpeedUp()) {
	  //speed up every 5 lines until very fast
	  lines+=tempint;
	  if(lines>5) {
	    lines=0;
	    if(level_tethover>100) {
	      level_tethover-=100;
	    } else {
	      //max speed tetrads
	    }
	  }
	}
	options->RuleFrequencyAddLines(tempint);
	tempint=500*(int)pow(2, tempint);
	sprintf(scoretext, "+%i", tempint);
	scoreblurb->SetText(scoretext);
	lastscoreblurb=lastupdate;
	score+=tempint;
      }
      sprintf(scoretext, "%08li", score);
      scoreboard->SetText(scoretext);
      scoreboard->Draw(screen);
      break;
    case GAMEOVER_SCREEN:
      draw_bg(true, screen, options);
      bucket->Draw(screen);
      scoreboard->Draw(screen);
      if(marathon) {
	draw_gameover(screen, SDL_GetTicks()-starttime);
      } else {
	draw_gameover(screen);
	timedisplay->SetText("00:00.00");
      }
      timedisplay->Draw(screen);
      while(SDL_PollEvent(&event)) {
	switch(event.type) {
	case SDL_QUIT:
	  done = 1;
	  break;
	case SDL_MOUSEBUTTONDOWN:
	case SDL_JOYBUTTONDOWN:
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  fakeinput.key.keysym.sym=SDLK_RETURN;
	  SDL_PushEvent(&fakeinput);
	  break;
	case SDL_KEYDOWN:
	  switch(event.key.keysym.sym) {
	  case SDLK_RETURN:
	  case SDLK_ESCAPE:
	  case SDLK_SPACE:
	    if(marathon) {
	      tempint=SCORE_MARATHON;
	    } else {
	      switch(timelength) {
	      case 120000:
		tempint=SCORE_2_MIN;
		break;
	      case 300000:
		tempint=SCORE_5_MIN;
		break;
	      case 600000:
		tempint=SCORE_10_MIN;
	      default:
		break;
	      }
	    }
	    if(score>options->GetHighScore(tempint, 9)) {
	      gothighscore=true;
	      display=HISCORES_SCREEN;
	    } else {
	      demostarted=SDL_GetTicks();
	      display=TITLE_SCREEN;
	    }
	    gameinprogress=false;
	    tetrad->Init(nexttetrad->GetType());
	    nexttetrad->Init(rand()%7);
	    bucket->Init(options->GetJunk());
	    ball->Init(options->GetSpeedBall());
	    paddle->Init(options->GetPadSizeX(), options->GetPadSizeY());
	    ballslost=bucketslost=0;
	    lines=0;
	    lastrulechange=SDL_GetTicks();
	    mainmenu->ModChoice("Start Game!", 1);
	    mainmenu->ModChoice("Rules/Difficulty...", 2);
	    mainmenu->SetState(4, NORMAL);
	    if(!marathon) {
	      mainmenu->SetState(5, NORMAL);
	    }
	    if(!marathon) {
	      mainmenu->SetState(6, NORMAL);
	    }
	    ruledisplay->SetText("Welcome to TONG!");
	    if(options->GetMusic()==MUSIC_SHUFFLE) {
	      Mix_FreeMusic(music);
	      if((music=Mix_LoadMUS(options->GetMusicFile()))) {
		Mix_PlayMusic(music, 1);
	      }
	    } else {
	      if(music) {
		Mix_PlayMusic(music, -1);
	      }
	    }
	    musicstarted=SDL_GetTicks();
	    break;
	  default:
	    break;
	  }
	  /*
	case SDL_MOUSEBUTTONDOWN:
	  fakeinput.type=SDL_KEYDOWN;
	  fakeinput.key.type=SDL_KEYDOWN;
	  fakeinput.key.state=SDL_PRESSED;
	  fakeinput.key.keysym.sym=SDLK_RETURN;
	  SDL_PushEvent(&fakeinput);
	  break;
	default:
	  break;
	  */
	}
      }
      break;
    case HISCORES_SCREEN:
      draw_bg(false, screen, options);
      if(gothighscore) {
	scoredisplay=tempint; //still set from gameover screen
	highscores->SetText(options->GetHighScoreText(scoredisplay));
      }
      highscores->Draw(screen);
      if(SDL_GetTicks()-demostarted>DEMO_TIME) {
	demostarted=SDL_GetTicks();
	switch(scoredisplay) {
	case SCORE_MARATHON:
	case SCORE_2_MIN:
	case SCORE_5_MIN:
	  scoredisplay++;
	  break;
	case SCORE_10_MIN:
	default:
	  scoredisplay=SCORE_MARATHON;
	  display=TITLE_SCREEN;
	  break;
	}
	highscores->SetText(options->GetHighScoreText(scoredisplay));
      } else if(gothighscore) {
	demostarted=SDL_GetTicks(); //stay in this screen 'til the user's done
	highscoremenu->Draw(screen);
      }
      while(SDL_PollEvent(&event)) {
	if(gothighscore) {
	  switch(event.type) {
	  case SDL_KEYDOWN:
	    if(event.key.keysym.sym==SDLK_RETURN) {
	      gothighscore=false; //all done
	      demostarted=SDL_GetTicks();
	      options->InsertScore(score, scorename, tempint);
	      if(!options->WriteConfig()) {
		printf("Settings were not saved.\n");
	      };
	      highscores->SetText(options->GetHighScoreText(scoredisplay));
	      highscoremenu->ModChoice("     _", 3);
	      strcpy(scorename, "_");
	      strcpy(scoretext, "--------");
	    } else if(event.key.keysym.sym==SDLK_LSHIFT ||
		      event.key.keysym.sym==SDLK_RSHIFT) {
	    } else if(event.key.keysym.sym==SDLK_BACKSPACE) {
	      scorename[strlen(scorename)-1]='\0';
	      strcpy(scoretext, "     ");
	      strcat(scoretext, scorename);
	      if(strlen(scorename)<5) {
		strcat(scoretext, "_");
	      }
	      highscoremenu->ModChoice(scoretext, 3);
	    } else if('a'<=(char)event.key.keysym.sym ||
		      (char)event.key.keysym.sym<='z') {
	      if(event.key.keysym.mod&(KMOD_LSHIFT|KMOD_RSHIFT)) {
		scoretext[0]=((char)event.key.keysym.sym)-32;
	      } else {
		scoretext[0]=(char)event.key.keysym.sym;
	      }
	      scoretext[1]='\0';
	      if(!strcmp(scorename, "_") || strlen(scorename)==0) {
		strcpy(scorename, scoretext);
	      } else {
		if(strlen(scorename)>=5) {
		  scorename[4]='\0';
		}
		strcat(scorename, scoretext);
	      }
	      strcpy(scoretext, "     ");
	      strcat(scoretext, scorename);
	      if(strlen(scorename)<5) {
		strcat(scoretext, "_");
	      }
	      highscoremenu->ModChoice(scoretext, 3);
	    }
	    break;
	  default:
	    break;
	  }
	} else {
	  switch(event.type) {
	  case SDL_QUIT:
	    done = 1;
	    break;
	  case SDL_KEYUP:
	    break;
	  default:
	    demostarted=SDL_GetTicks();
	    display=TITLE_SCREEN;
	    break;
	  }
	}
      }
      break;
    default:
      break;
    }
    SDL_Flip(screen);
    lastlastupdate=lastupdate;
    lastupdate=SDL_GetTicks();
  }  //main game loop

  if(Mix_PlayingMusic()) {
    Mix_FadeOutMusic(1000);
    Mix_FreeMusic(music);
  }
  if(joy) {
    SDL_JoystickClose(joy);
  }
  if(options->WriteConfig()) {
  } else {
    printf("Settings could not be saved\n");
  }

  Mix_CloseAudio();
  delete tetrad;
  delete nexttetrad;
  delete bucket;
  delete ball;
  delete paddle;
  delete options;
  delete scoreboard;
  delete scoreblurb;
  delete ruledisplay;
  delete timedisplay;
  delete highscores;
  delete highscoremenu;
  delete mainmenu;
  SDL_Quit();

  return 0;
}


bool drop_piece(Tetrad* tetrad, Tetrad* nexttetrad, Bucket* bucket,
		Uint32& lastpush, bool& newdown, bool& marathon, Option* ops,
		int& display) {
  bool lostbucket=false;
  if(!tetrad->MoveDown(bucket)) {
    if(!tetrad->GetMoves()) {
      if(marathon) {
	display=GAMEOVER_SCREEN;
	bucket->GreyOut();
	Mix_FadeOutMusic(1000);
	if(ops->GetSoundFx()) {
	  Mix_PlayChannel(2, twobits, 0);
	}
      } else {
	//      Mix_PlayChannel(0, bust, 0);  //TODO:  bucket reset sound
	//      bucket->Init(rand()%5);
	bucket->StartOver();
	lostbucket=true;
      }
    }
    if(!bucket->IsBreaking()) {
      tetrad->Init(nexttetrad->GetType());
      nexttetrad->Init(rand()%7);
      if(ops->GetSoundFx()) {
	Mix_PlayChannel(0, plop, 0);
      }
    }
    newdown=true;
  }
  lastpush=SDL_GetTicks();
  return lostbucket;
}
